#include <avr/io.h>
#include "i2c_eep.h"

/*****************************
*  Routine wurde an EEPROM   *
*  vom Typ 24C256 angepasst  *
*****************************/


unsigned char i2c_eep_start ( void );
unsigned char i2c_eep_stop ( void );
unsigned char i2c_eep_write_byte ( unsigned char data );
unsigned int i2c_eep_read_byte ( void );


/* Schreibroutine */
extern unsigned int i2c_eep_write ( unsigned int Addr, unsigned char Data )
{
 if ( i2c_eep_start() ) { return 0x0100; } // Abbruch, Busfehler!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_WRITE ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( (char) Addr >> 8 ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( (char) Addr ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( Data ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_stop() ) { return 0x0100; } // Abbruch, Busfehler!
 return 0;
}


/* Leseroutine */
extern unsigned int i2c_eep_read ( unsigned int Addr )
{
 unsigned int data;
 if ( i2c_eep_start() ) { return 0x0100; } // Abbruch, Busfehler!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_WRITE ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( (char) Addr >> 8 ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 if ( i2c_eep_write_byte( (char) Addr ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 SDA_HIGH; SCL_HIGH; // Re-Start vorbereiten
 if ( i2c_eep_start() ) { return 0x0200; } // Abbruch, Bus belegt!
 if ( i2c_eep_write_byte( I2C_DEVICE + I2C_READ ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 data = i2c_eep_read_byte();
 if ( i2c_eep_stop() ) { return 0x0100; } // Abbruch, Busfehler!
 return data;
}

/* Start_Routine */
unsigned char i2c_eep_start ( void )
{
 SDA_IN;
 SCL_IN;
 if ( !( SDA_READ && SCL_READ ) ) { return 1; } // Abbruch => Bus nicht bereit!
 SDA_OUT;
 SDA_LOW;
 SCL_OUT;
 SCL_LOW;
 return 0;
}


/* Stop_Routine */
unsigned char i2c_eep_stop( void )
{
 SCL_HIGH;
 SDA_OUT;
 SDA_HIGH;
 SCL_IN;
 SDA_IN;
 if ( !( SDA_READ && SCL_READ ) ) { return 1; } // Abbruch => Busfehler!
 return 0;
}


/* Einzelnes Byte schreiben */
unsigned char i2c_eep_write_byte ( unsigned char data )
{
 unsigned char mask = 0x80, i;
 for ( i=0; i<8; i++)
 {
  if ( data & mask )
  {
   // SDA-High ausgeben
   SDA_HIGH;
   SCL_HIGH;
   SCL_LOW;
  }
  else
  {
   // SDA-Low ausgeben
   SDA_LOW;
   SCL_HIGH;
   SCL_LOW;
  }
  mask >>= 1;
 }
 SDA_IN;
 SCL_HIGH;
 if ( SDA_READ ) { return 1; } // Abbruch, Acknowledge falsch
 SCL_LOW;
 SDA_OUT;
 SDA_LOW;
 return 0;
}


/* Einzelnes Byte lesen */
unsigned int i2c_eep_read_byte ( void )
{
 unsigned char mask = 0x80, i, data = 0x00;
 SDA_IN;
 for ( i=0; i<8; i++)
 {
  SCL_HIGH;
  if ( SDA_READ ) { data |= mask; } // SDA = 1 => sichern!
  SCL_LOW;
  mask >>= 1;
 }
 SCL_HIGH;
 if ( !( SDA_READ ) ) { return 0x0200; } // Abbruch, Acknowledge falsch
 SCL_LOW;
 return data;
}

